@@ -68,22 +68,27 @@ def _argv(*hosts, **variables):
 
     hosts = hosts or ('localhost',)
 
-    inv = []
     user = None
-    for host in hosts:
-        if '@' in host:
-            parts = host.split('@')
-            if parts[0]:
-                user = parts[0]
-            inv.append(parts[1])
-        else:
-            inv.append(host)
+    inv_arg = any([
+        a.startswith('--inventory') or a.startswith('-i')
+        for a in argv
+    ])
+    inv = []
+    if not inv_arg:
+        for host in hosts:
+            if '@' in host:
+                parts = host.split('@')
+                if parts[0]:
+                    user = parts[0]
+                inv.append(parts[1])
+            else:
+                inv.append(host)
 
     ssh_arg = any([a.startswith('--ssh-extra-args') for a in argv])
 
     if inv == ['localhost']:
         argv += ['-c', 'local']
-    elif len(inv) == 1 and not ssh_arg:
+    elif (inv_arg or len(inv) == 1) and not ssh_arg:
         ssh = {}
         ssh['ControlMaster'] = 'auto'
         ssh['ControlPersist'] = '120s'
@@ -97,13 +102,18 @@ def _argv(*hosts, **variables):
     if user:
         argv += ['-u', user]
 
-    if inv:
+    if inv and not inv_arg:
         argv += ['-i', ','.join(inv) + ',']
 
     for key, value in variables.items():
         if not isinstance(value, str):
-            value = json.dumps(value)
-        argv += ['-e', key + '=' + shlex.quote(value)]
+            value = shlex.quote(json.dumps(value))
+        # apparently proper proxy quoted values such as foo="'+ bar'"
+        elif value.startswith('"'):
+            value = "'" + value + "'"
+        elif value.startswith("'"):
+            value = '"' + value + '"'
+        argv += ['-e', key + '=' + value]
 
     return argv
 
@@ -245,7 +255,7 @@ def role(role, *hosts, **variables):
         stdout=sys.stdout,
     )
     pp.communicate()
-    return pp.returncode
+    sys.exit(pp.returncode)
 
 
 def tasks(tasks, *hosts, **variables):
@@ -275,7 +285,7 @@ def tasks(tasks, *hosts, **variables):
         stdout=sys.stdout,
     )
     p.communicate()
-    return p.returncode
+    sys.exit(p.returncode)
 
 
 def playbook(playbook, *hosts, **variables):
@@ -298,7 +308,7 @@ def playbook(playbook, *hosts, **variables):
         stdout=sys.stdout,
     )
     p.communicate()
-    return p.returncode
+    sys.exit(p.returncode)
 
 
 def run(source, *hosts_or_tasks, **variables):
@@ -322,7 +332,7 @@ def run(source, *hosts_or_tasks, **variables):
     hosts = []
     tasks = []
     for arg in hosts_or_tasks:
-        if '@' in arg:
+        if '@' in arg and ' ' not in arg:
             hosts.append(arg)
         else:
             tasks.append(arg)
